# (c) 2002 Tony Monroe
package NetHirc::Terminal;

use strict;
use vars qw( @ISA );

use Carp;
use IO::Handle;
use IO::File;

use NetHirc::BigBrother;
use NetHirc::Channel;
use NetHirc::Formats;
use NetHirc::Helpers;
use NetHirc::Server;
use NetHirc::State;
use NetHirc::TerminalEvent;

my $ssfe = $ENV{'SSFE'};

@ISA = qw(IO::Handle);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $handle = shift;
    my $self = bless $handle, $class;
    $self->init(@_);
    return $self;
}

sub param {
    my $self = shift;
    my $param = shift;
    if (@_) {
	${*$self}{"nethirc_terminal_$param"} = $_[0];
    }
    return ${*$self}{"nethirc_terminal_$param"};
}

sub unparam {
    my $self = shift;
    my $param = shift;
    delete ${*$self}{"nethirc_terminal_$param"};
}

sub init {
    my $self = shift;
    my %args = @_;
    $self->param('cmdchar', $args{'CmdChar'} || '/');
    $self->param('output', $args{'Output'} || \*STDOUT);
    $self->param('complaints_file', $args{'Complaints'});
    $self->gather_complaints();
    $self->param('status', $args{'Status'} || $irc_formats{'status'});
    $self->param('totalitarian', $args{'Totalitarian'} || 0);
    $self->param('happyslash', 0);
    my $secs = 60 - (localtime(time()))[0];
    if ($self->param('totalitarian')) {
	$self->init_bigbrother();
    }
    $self->param('output')->autoflush(1);
}

sub init_bigbrother {
    my $self = shift;
    my $bigbrother = new NetHirc::BigBrother();
    $bigbrother->set_announce(\&bigbrother_announce);
    $self->param('bigbrother', $bigbrother);
}

sub spit {
    my $self = shift;
    if ($self->eof()) {
	$self->close();
	return;
    }
    $self->process($self->getline());
}

sub print {
    my $self = shift;
    $self->param('output')->print(@_);
}

sub printf {
    my $self = shift;
    $self->param('output')->printf(@_);
}

##
## %%	%
## %C	current channel
## %N	your nickname on this server
## %S	current server
## %T 	24 hour time
## %H	12 hour time
## 

sub cook
{
    my $self = shift;
    my $cooked = shift;
    chomp $cooked;
    my ($sec, $min, $hour) = localtime(time());
    my $hour12 = ((not ($hour % 12)) ? 12 : ($hour % 12));
    my $t12h = sprintf("%02d:%02d", $hour12, $min);
    my $t24h = sprintf("%02d:%02d", $hour, $min);
    my $srv = NetHirc::Server->current();
    my $chobj = NetHirc::Channel->current();
    my $conn;
    my $nickname;
    my $server;
    my $channel;
    if ($srv) {
	$conn = $srv->conn();
	$nickname = $conn->nick;
	$server = ($conn->connected() ? $conn->server() : "no server");
	$channel = $chobj ? $chobj->name() : "no channel";
    } else {
	$nickname = "Amnesiac";
	$server = "no server";
	$channel = "no channel";
    }
    $cooked =~ s/%C/$channel/g;
    $cooked =~ s/%N/$nickname/g;
    $cooked =~ s/%S/$server/g;
    $cooked =~ s/%H/$t12h/g;
    $cooked =~ s/%T/$t24h/g;
    return $cooked;
}

sub process {
    my $self = shift;
    my $text = shift;
    my $cmdchar = $self->param('cmdchar');
    my $event;
    my $query;
    chomp $text;
    if (substr($text, 0, 6) eq '@ssfe@') {
	$ssfe = 1;
	return;
    } elsif ($text =~ /^\s*$/) {
	$event = new NetHirc::TerminalEvent(Type => 'empty');
    } elsif ($text =~ /^$cmdchar(\S+)(\s*)(.*)/) {
	my $type = $1;
	my $space = $2;
	my $args = $3;
	if (ref NetHirc::TerminalEvent->cmd($type) ne 'CODE') {
	    if ($self->param('happyslash')) {
		my $ev0 = new NetHirc::TerminalEvent(Type => 'slashhappy', Args => undef);
		$ev0->handler($self);
		$event = new NetHirc::TerminalEvent(Type => 'say', Args => "$cmdchar$type$space$args");
	    } else {
		$event = new NetHirc::TerminalEvent(Type => 'error', Args => $type);
	    }
	} else {
	    $event = new NetHirc::TerminalEvent(Type => $type, Args => $args);
	}
    } else {
	if ($self->param('confirm_answer')) {
	    if ($text eq $self->param('confirm_answer')) {
		$self->param('confirm_code')->();
	    }
	    $self->unparam('confirm_answer');
	    $self->unparam('confirm_code');
	    return;
	} else {
	    $event = new NetHirc::TerminalEvent(Type => 'say', Args => $text);
	}
    }
    $event->handler($self);
}

sub add_handler {
    my $self = shift;
    NetHirc::TerminalEvent::add_handler(@_);
}

sub remove_handler {
    my $self = shift;
    NetHirc::TerminalEvent::remove_handler(@_);
}

sub gather_complaints {
    my $self = shift;
    my $complaints_file = $self->param('complaints_file');
    my @complaints = ();
    my $cfh = new IO::File($complaints_file, "r");
    unless ($complaints_file && $cfh) {
	push @complaints, "Blank lines are dull.";
	$self->param('complaints', \@complaints);
	return;
    }
    @complaints = $cfh->getlines();
    $cfh->close();
    chomp @complaints;
    $self->param('complaints', \@complaints);
    return;
}

sub complain {
    my $self = shift;
    my $complaint = NetHirc::Helpers::pickrandom(@{$self->param('complaints')});
    $self->printf(ircformat('complaint', $complaint));
}

sub display_status_line {
    return unless $ssfe;
    my $self = shift;
    my $status = $self->param('status');
    my $cooked = $self->cook($status);
    $self->printf("`#ssfe#s$cooked\n");
}

sub bigbrother_announce {
    my $bb = shift;
    my $terminal = $NetHirc::State::terminal;
    my @verboten = map { qq("$_") } $bb->verboten();
    $terminal->print(ircformat('verboten0'));
    $terminal->print(ircformat('verboten1', join("  ", @verboten)));
    $terminal->print(ircformat('verboten2'));
    $terminal->print(ircformat('verboten3'));
}

sub confirm {
    my $self = shift;
    my $question = shift;
    my $wanted_answer = shift;
    my $if_yes = shift;
    if (ref $if_yes ne 'CODE') {
	croak __PACKAGE__, "::confirm -- not a CODE ref";
    }
    $self->param('confirm_answer', $wanted_answer);
    $self->param('confirm_code', $if_yes);
    $self->print($question);
}


1;
__END__
SSFE quick reference:

`#ssfe#i	set irc mode, confirmed with @ssfe@i
`#ssfe#c	set cooked mode, confirmed with @ssfe@c
`#ssfe#s<text>	set status line to <text>
`#ssfe#T<text>	set text sent with ^T (newline added)
`#ssfe#t<text>	add alternative to tab list
`#ssfe#l	clear screen
`#ssfe#P	prompt something, noecho
`#ssfe#p	prompt something, echo
`#ssfe#n<text>	insert text on input line
`#ssfe#o<text>	store text to be recalled to input line via ^O
